﻿using System.Collections.ObjectModel;
using System.Net.Http;
using System.Text.Json;
using System.Windows;

namespace DyuClient.ViewPages.TabPages
{
    public partial class Demo3
    {
        // 将HttpClient提取为类级别的成员，避免重复创建实例，并设置超时时间（示例设置为30秒）
        private static readonly HttpClient client = new HttpClient
        {
            Timeout = TimeSpan.FromSeconds(30)
        };

        private ObservableCollection<HttpPost> dataList;

        public Demo3()
        {
            InitializeComponent();
            dataList = new ObservableCollection<HttpPost>();
        }

        /// <summary>
        /// 按钮点击事件，触发Http请求、数据解析以及绑定到表格
        /// </summary>
        private async void GetDataButton_Click(object sender, RoutedEventArgs e)
        {
            // 调用异步方法获取JSON数据并解析
            var result = await GetJsonDataAsync("https://httpbin.org/get");

            // 清空之前的数据（如果有）
            dataList.Clear();

            // 根据解析结果填充数据到dataList
            if (result!= null)
            {
                dataList.Add(result);
            }

            // 将获取到的数据绑定到DataGridTable
            DataGridTable.ItemsSource = dataList;
        }

        /// <summary>
        /// 异步方法：发送Http请求并解析JSON，添加了更完善的错误处理和缓存逻辑（此处假设添加简单内存缓存示例）
        /// </summary>
        private async Task<HttpPost> GetJsonDataAsync(string url)
        {
            // 检查是否已经有缓存数据（此处简单示例，实际可根据需求完善缓存机制）
            HttpPost cachedData = CheckCacheForData(url);
            if (cachedData!= null)
            {
                return cachedData;
            }

            try
            {
                var response = await client.GetAsync(url);
                response.EnsureSuccessStatusCode();

                string jsonResponse = await response.Content.ReadAsStringAsync();

                // 输出获取到的原始JSON数据，方便调试查看是否获取正确
                Console.WriteLine("获取到的JSON数据: " + jsonResponse);

                // 反序列化为HttpPost对象
                var post = JsonSerializer.Deserialize<HttpPost>(jsonResponse);

                // 输出反序列化后的对象信息，查看是否反序列化成功
                Console.WriteLine("反序列化后的HttpPost对象: " + post);

                // 将获取到的数据存入缓存（此处简单示例，实际可根据需求完善缓存有效期等设置）
                AddToCache(url, post);

                return post;
            }
            catch (HttpRequestException httpEx) when (httpEx.StatusCode == System.Net.HttpStatusCode.NotFound)
            {
                MessageBox.Show($"请求的资源未找到: {httpEx.Message}");
                return null;
            }
            catch (HttpRequestException httpEx) when (httpEx.StatusCode == System.Net.HttpStatusCode.Unauthorized)
            {
                MessageBox.Show($"未授权访问，请检查认证信息: {httpEx.Message}");
                return null;
            }
            catch (HttpRequestException httpEx)
            {
                MessageBox.Show($"网络请求错误: {httpEx.Message}");
                return null;
            }
            catch (Exception ex)
            {
                MessageBox.Show($"请求失败: {ex.Message}");
                return null;
            }
        }

        /// <summary>
        /// 简单的内存缓存检查方法示例，实际可根据需求完善缓存机制
        /// </summary>
        private HttpPost CheckCacheForData(string url)
        {
            // 此处可添加实际的缓存检查逻辑，比如从内存缓存字典中查找对应URL的缓存数据
            // 假设这里有一个简单的字典作为缓存存储（实际应用中可能需要更完善的缓存管理类等）
            // Dictionary<string, HttpPost> cacheDictionary = new Dictionary<string, HttpPost>();
            // if (cacheDictionary.ContainsKey(url))
            // {
            //     return cacheDictionary[url];
            // }
            return null;
        }

        /// <summary>
        /// 简单的内存缓存添加方法示例，实际可根据需求完善缓存机制
        /// </summary>
        private void AddToCache(string url, HttpPost data)
        {
            // 此处可添加实际的缓存添加逻辑，比如将数据存入内存缓存字典
            // Dictionary<string, HttpPost> cacheDictionary = new Dictionary<string, HttpPost>();
            // cacheDictionary[url] = data;
        }

        /// <summary>
        /// 刷新按钮的点击事件处理方法
        /// </summary>
        private async void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            dataList.Clear();
            // 这里可以复用获取数据的逻辑，重新获取并更新数据
            var result = await GetJsonDataAsync("https://httpbin.org/get");

            if (result!= null)
            {
                dataList.Add(result);
            }

            DataGridTable.ItemsSource = dataList;
        }

        /// <summary>
        /// 清空按钮的点击事件处理方法
        /// </summary>
        private void ClearButton_Click(object sender, RoutedEventArgs e)
        {
            dataList.Clear();
            DataGridTable.ItemsSource = dataList;
        }
    }
}